@@ -34,13 +34,6 @@
 #define CMD_NAME "icainfo"
 #define COPYRIGHT "Copyright IBM Corp. 2007, 2016."
 
-#define CARD_AVAILABLE		0x01
-#define CEXnA_AVAILABLE		0x02
-#define CEXnC_AVAILABLE		0x04
-#define CEX4C_AVAILABLE		0x08
-#define CEXnP_AVAILABLE		0x10
-
-
 void print_version(void)
 {
 	printf(CMD_NAME ": libica version " VERSION "\n" COPYRIGHT "\n");
@@ -76,6 +69,8 @@ static struct crypt_pair crypt_map[] = {
 	{"SHA-256", SHA256},
 	{"SHA-384", SHA384},
 	{"SHA-512", SHA512},
+	{"SHA-512/224", SHA512_224},
+	{"SHA-512/256", SHA512_256},
 	{"SHA3-224", SHA3_224},
 	{"SHA3-256", SHA3_256},
 	{"SHA3-384", SHA3_384},
@@ -89,6 +84,16 @@ static struct crypt_pair crypt_map[] = {
 	{"ECDSA Sign", EC_DSA_SIGN},
 	{"ECDSA Verify", EC_DSA_VERIFY},
 	{"ECKGEN", EC_KGEN},
+	{"Ed25519 Keygen", ED25519_KEYGEN},
+	{"Ed25519 Sign", ED25519_SIGN},
+	{"Ed25519 Verify", ED25519_VERIFY},
+	{"Ed448 Keygen", ED448_KEYGEN},
+	{"Ed448 Sign", ED448_SIGN},
+	{"Ed448 Verify", ED448_VERIFY},
+	{"X25519 Keygen", X25519_KEYGEN},
+	{"X25519 Derive", X25519_DERIVE},
+	{"X448 Keygen", X448_KEYGEN},
+	{"X448 Derive", X448_DERIVE},
 	{"RSA ME", RSA_ME},
 	{"RSA CRT", RSA_CRT},
 	{"DES ECB", DES_ECB},
@@ -115,64 +120,13 @@ static struct crypt_pair crypt_map[] = {
 };
 
 
-static int search_for_cards()
-{
-	int ret=0, nr, version;
-	char type, fname[80], buf[80];
-	FILE *f;
-
-	for (nr = 0; nr <= 0xFF; nr++) {
-		snprintf(fname, sizeof(fname), "/sys/devices/ap/card%02x/type", nr);
-		f = fopen(fname, "r");
-		if (f) {
-			if (fgets(buf, sizeof(buf), f)) {
-				version = 0;
-				type = ' ';
-				sscanf(buf + 3, "%d%c", &version, &type);
-				ret = ret | CARD_AVAILABLE;
-				if (type == 'A') {
-					ret = ret | CEXnA_AVAILABLE;
-				} else if (type == 'C') {
-					ret = ret | CEXnC_AVAILABLE;
-					if (version >= 4) {
-						ret = ret | CEX4C_AVAILABLE;
-					}
-				} else if (type == 'P') {
-					ret = ret | CEXnP_AVAILABLE;
-				}
-			}
-			fclose(f);
-		}
-	}
-
-	return ret;
-}
-
-static inline int card_available(unsigned int flags)
-{
-	if (flags)
-		return 1;
-	else
-		return 0;
-}
-
-static inline int cex4c_available(unsigned int flags)
-{
-	if ((flags & CEX4C_AVAILABLE) == CEX4C_AVAILABLE)
-		return 1;
-	else
-		return 0;
-}
-
-
 int main(int argc, char **argv)
 {
 	int rc;
 	int index = 0;
 	unsigned int mech_len, j;
 	libica_func_list_element *pmech_list = NULL;
-	int flags;
-	int i;
+	unsigned int i;
 
 	while ((rc = getopt_long(argc, argv, getopt_string,
 				 getopt_long_options, &index)) != -1) {
@@ -197,8 +151,8 @@ int main(int argc, char **argv)
 		exit(1);
 	}
 
-	printf("      Cryptographic algorithm support      \n");
-	printf("-------------------------------------------\n");
+	printf("              Cryptographic algorithm support      \n");
+	printf("------------------------------------------------------\n");
 
 	if (ica_get_functionlist(NULL, &mech_len) != 0){
 		perror("get_functionlist: ");
@@ -211,12 +165,11 @@ int main(int argc, char **argv)
 		return EXIT_FAILURE;
 	}
 
-	flags = search_for_cards();
-
 	#define CELL_SIZE 3
 
-	printf(" function      |  hardware  |  software  \n");
-	printf("---------------+------------+------------\n");
+	printf("               |         hardware        |            \n");
+	printf(" function      |   dynamic  |   static   |  software  \n");
+	printf("---------------+------------+------------+------------\n");
 	for(i = 0; crypt_map[i].algo_id; i++){
 		for(j = 0; j < mech_len; j++){
 			if(crypt_map[i].algo_id == pmech_list[j].mech_mode_id){
@@ -224,71 +177,25 @@ int main(int argc, char **argv)
 				if (((ica_fips_status() & ICA_FIPS_MODE)
 				    && !fips_approved(pmech_list[j].mech_mode_id))
 				    || ica_fips_status() >> 1) {
-					printf("%14s |  blocked   "
-						"|   blocked\n",
+					printf("%14s |  blocked   |   blocked  |   blocked\n",
 						crypt_map[i].name);
 					break;
 				}
 #endif /* ICA_FIPS */
-
-				if (crypt_map[i].algo_id == EC_DH ||
-					crypt_map[i].algo_id == EC_DSA_SIGN ||
-					crypt_map[i].algo_id == EC_DSA_VERIFY ||
-					crypt_map[i].algo_id == EC_KGEN) {
-					/* Functions that need a CEX4C or later */
-					if (cex4c_available(flags)) {
-						printf("%14s |    %*s     |     %*s\n",
-							crypt_map[i].name,
-							CELL_SIZE,
-							pmech_list[j].flags &
-							(ICA_FLAG_SHW | ICA_FLAG_DHW)
-							? "yes" : "no",
-							CELL_SIZE,
-							pmech_list[j].flags & ICA_FLAG_SW
-							? "yes" : "no");
-					} else {
-						printf("%14s |    %*s     |     %*s\n",
-							crypt_map[i].name,
-							CELL_SIZE,
-							pmech_list[j].flags &
-							(ICA_FLAG_SHW)
-							? "yes" : "no",
-							CELL_SIZE,
-							pmech_list[j].flags & ICA_FLAG_SW
-							? "yes" : "no");
-					}
-
-				} else if (card_available(flags)) {
-					/* Functions that need any card */
-					printf("%14s |    %*s     |     %*s\n",
-						crypt_map[i].name,
-						CELL_SIZE,
-						pmech_list[j].flags &
-						(ICA_FLAG_SHW | ICA_FLAG_DHW)
-						? "yes" : "no",
-						CELL_SIZE,
-						pmech_list[j].flags & ICA_FLAG_SW
-						? "yes" : "no");
-				} else {
-					printf("%14s |    %*s     |     %*s\n",
-						crypt_map[i].name,
-						CELL_SIZE,
-						(pmech_list[j].flags &
-						ICA_FLAG_SHW)
-						? "yes" : "no",
-						CELL_SIZE,
-						pmech_list[j].flags & ICA_FLAG_SW
-						? "yes" : "no");
-
-				}
-				break;
+				printf("%14s |    %*s     |    %*s     |    %*s\n",
+					crypt_map[i].name,
+					CELL_SIZE,
+					pmech_list[j].flags & ICA_FLAG_DHW ? "yes" : "no",
+					CELL_SIZE,
+					pmech_list[j].flags & ICA_FLAG_SHW ? "yes" : "no",
+					CELL_SIZE,
+					pmech_list[j].flags & ICA_FLAG_SW ? "yes" : "no");
 			}
-
 		}
 	}
 	free(pmech_list);
+	printf("------------------------------------------------------\n");
 
-	printf("-------------------------------------------\n");
 #ifdef ICA_FIPS
 	printf("Built-in FIPS support: FIPS mode %s.\n",
 	    ica_fips_status() & ICA_FIPS_MODE ? "active" : "inactive");
