@@ -25,6 +25,7 @@
 #include <fcntl.h>
 #include <dirent.h>
 #include "icastats.h"
+#include "init.h"
 
 #define NOT_INITIALIZED (-1)
 #define NAME_LENGHT 20
@@ -33,8 +34,7 @@ static stats_entry_t *stats = NULL;
 volatile int stats_shm_handle = NOT_INITIALIZED;
 
 
-
-static void atomic_add(int *x, int i)
+static inline void atomic_add(int *x, int i)
 {
 	int old;
 	int new;
@@ -62,9 +62,11 @@ static void atomic_add(int *x, int i)
 
 int stats_mmap(int user)
 {
+	char shm_id[NAME_LENGHT];
+
 	if (stats == NULL) {
-		char shm_id[NAME_LENGHT];
-		sprintf(shm_id, "icastats_%d", user == -1? geteuid(): user);
+		sprintf(shm_id, "icastats_%d",
+			user == -1 ? geteuid() : (uid_t)user);
 
 		stats_shm_handle = shm_open(shm_id, O_CREAT | O_RDWR,
 						    S_IRUSR | S_IWUSR);
@@ -256,6 +258,7 @@ char *get_next_usr()
 	return NULL;
 }
 
+#ifndef ICASTATS
 /* increments a field of the shared memory segment
  * arguments:
  * @field - the enum of the field see icastats.h
@@ -264,9 +267,11 @@ char *get_next_usr()
  * @direction - valid values are ENCRYPT and DECRYPT
  */
 
-
 void stats_increment(stats_fields_t field, int hardware, int direction)
 {
+	if (!ica_stats_enabled)
+		return;
+
 	if (stats == NULL)
 		return;
 
@@ -281,6 +286,7 @@ void stats_increment(stats_fields_t field, int hardware, int direction)
 		else
 			atomic_add((int *)&stats[field].dec.sw, 1);
 }
+#endif
 
 
 /* Reset the shared memory segment to zero
