@@ -35,6 +35,8 @@ typedef enum stats_fields {
 	ICA_STATS_SHA256,
 	ICA_STATS_SHA384,
 	ICA_STATS_SHA512,
+	ICA_STATS_SHA512_224,
+	ICA_STATS_SHA512_256,
 	ICA_STATS_SHA3_224,
 	ICA_STATS_SHA3_256,
 	ICA_STATS_SHA3_384,
@@ -44,6 +46,20 @@ typedef enum stats_fields {
 	ICA_STATS_GHASH,
 	ICA_STATS_PRNG,
 	ICA_STATS_DRBGSHA512,
+	ICA_STATS_ECDH,
+	ICA_STATS_ECDSA_SIGN,
+	ICA_STATS_ECDSA_VERIFY,
+	ICA_STATS_ECKGEN,
+	ICA_STATS_ED25519_KEYGEN,
+	ICA_STATS_ED25519_SIGN,
+	ICA_STATS_ED25519_VERIFY,
+	ICA_STATS_ED448_KEYGEN,
+	ICA_STATS_ED448_SIGN,
+	ICA_STATS_ED448_VERIFY,
+	ICA_STATS_X25519_KEYGEN,
+	ICA_STATS_X25519_DERIVE,
+	ICA_STATS_X448_KEYGEN,
+	ICA_STATS_X448_DERIVE,
 	ICA_STATS_RSA_ME,
 	ICA_STATS_RSA_CRT, /* add new crypt counters above RSA_CRT
 			      (see print_stats function) */
@@ -80,6 +96,8 @@ typedef enum stats_fields {
 	"SHA-256",    	\
 	"SHA-384",    	\
 	"SHA-512",    	\
+	"SHA-512/224",	\
+	"SHA-512/256",	\
 	"SHA3-224",    	\
 	"SHA3-256",    	\
 	"SHA3-384",    	\
@@ -89,6 +107,20 @@ typedef enum stats_fields {
 	"GHASH",      	\
 	"P_RNG",      	\
 	"DRBG-SHA-512",	\
+	"ECDH",         \
+	"ECDSA Sign",   \
+	"ECDSA Verify", \
+	"ECKGEN",       \
+	"Ed25519 Keygen",\
+	"Ed25519 Sign", \
+	"Ed25519 Verify",\
+	"Ed448 Keygen",\
+	"Ed448 Sign", \
+	"Ed448 Verify",\
+	"X25519 Keygen",\
+	"X25519 Derive",\
+	"X448 Keygen",  \
+	"X448 Derive",  \
 	"RSA-ME",     	\
 	"RSA-CRT",    	\
 	"DES ECB",    	\
