@@ -33,20 +33,21 @@
 #define AES_BLOCK_SIZE 16
 #define GCM_RECOMMENDED_IV_LENGTH 12
 
-#define HS_FLAG 	0x400;
-#define LAAD_FLAG 	0x200;
-#define LPC_FLAG 	0x100;
+#define HS_FLAG		0x400;
+#define LAAD_FLAG	0x200;
+#define LPC_FLAG	0x100;
 
 
 static inline int s390_aes_gcm_hw(unsigned int function_code,
-				  const unsigned char *input_data, unsigned char *output_data,
+				  const unsigned char *input_data,
+				  unsigned char *output_data,
 				  unsigned long input_length,
-				  unsigned char *key,
-				  unsigned char *j0, unsigned long j0_width,
-				  unsigned char *ctr, unsigned long ctr_width,
-				  const unsigned char *aad, unsigned long aad_length,
+				  unsigned char *key, unsigned char *j0,
+				  unsigned char *ctr,
+				  const unsigned char *aad,
+				  unsigned long aad_length,
 				  const unsigned char *subkey_h,
-				  unsigned char *tag, unsigned long tag_length,
+				  unsigned char *tag,
 				  unsigned int laad, unsigned int lpc)
 {
 	struct {
@@ -60,7 +61,7 @@ static inline int s390_aes_gcm_hw(unsigned int function_code,
 		ica_aes_key_len_256_t key;
 	} parm_block;
 
-	unsigned int rc = 0;
+	int rc;
 	unsigned int key_size = (function_code & 0x0f) * sizeof(ica_aes_key_single_t);
 
 	memset(&parm_block, 0, sizeof(parm_block));
@@ -69,17 +70,19 @@ static inline int s390_aes_gcm_hw(unsigned int function_code,
 	memcpy(&parm_block.key, key, key_size);
 
 	if (laad && lpc) {
-		parm_block.total_aad_length = aad_length*8; // total length in bits
-		parm_block.total_input_length = input_length*8; // total length in bits
+		// total length in bits
+		parm_block.total_aad_length = aad_length * 8;
+
+		// total length in bits
+		parm_block.total_input_length = input_length * 8;
+
 		parm_block.cv = input_length / AES_BLOCK_SIZE + 1;
 	}
 
 	if (ctr) {
-		memcpy(&parm_block.cv, &ctr[GCM_RECOMMENDED_IV_LENGTH], sizeof(int));
-		memcpy(&parm_block.j0, ctr, AES_BLOCK_SIZE); //GCM_RECOMMENDED_IV_LENGTH);
-		//unsigned int* cv;
-		//cv = (unsigned int*)&(parm_block.j0[GCM_RECOMMENDED_IV_LENGTH]);
-		//*cv = 1;
+		memcpy(&parm_block.cv, &ctr[GCM_RECOMMENDED_IV_LENGTH],
+		       sizeof(parm_block.cv));
+		memcpy(&parm_block.j0, ctr, AES_BLOCK_SIZE);
 	}
 
 	if (j0)
@@ -99,47 +102,47 @@ static inline int s390_aes_gcm_hw(unsigned int function_code,
 	if (input_length == 0 && aad_length == 0)
 		parm_block.cv++;
 
-	rc = s390_kma(function_code, &parm_block,
-			  output_data, input_data, input_length,
-			  aad, aad_length);
+	rc = s390_kma(function_code, &parm_block, output_data, input_data,
+		      input_length, aad, aad_length);
 
 	if (rc >= 0) {
 		memcpy(tag, &parm_block.tag, AES_BLOCK_SIZE);
-		if (ctr)
-			memcpy(&ctr[GCM_RECOMMENDED_IV_LENGTH], &parm_block.cv, sizeof(int)); // not in last call
+		if (ctr) {
+			// not in last call
+			memcpy(&ctr[GCM_RECOMMENDED_IV_LENGTH], &parm_block.cv,
+			       sizeof(parm_block.cv));
+		}
 		return 0;
-	} else
+	} else {
 		return EIO;
+	}
 }
 
 static inline int s390_aes_gcm(unsigned int fc, const unsigned char *in_data,
 			unsigned char *out_data, unsigned long data_length,
-			unsigned char *key,
-			unsigned char *j0, unsigned int j0_width,
-			unsigned char *ctr, unsigned int ctr_width,
+			unsigned char *key, unsigned char *j0,
+			unsigned char *ctr,
 			const unsigned char *aad, unsigned long aad_length,
 			unsigned char *subkey_h, unsigned char *tag,
-			unsigned long tag_length, unsigned int laad,
-			unsigned int lpc)
+			unsigned int laad, unsigned int lpc)
 {
-	int rc = 1;
-	int hardware = ALGO_HW;
+	int rc = ENODEV;
 
-	if (*s390_kma_functions[fc].enabled)
-		rc = s390_aes_gcm_hw(s390_kma_functions[fc].hw_fc,
-				     in_data, out_data, data_length,
-				     key, j0, j0_width, ctr, ctr_width, aad, aad_length,
-				     subkey_h, tag, tag_length, laad, lpc);
+	if (*s390_kma_functions[fc].enabled) {
+		rc = s390_aes_gcm_hw(s390_kma_functions[fc].hw_fc, in_data,
+				     out_data, data_length, key, j0, ctr, aad,
+				     aad_length, subkey_h, tag, laad, lpc);
+	}
 	if (rc)
 		return rc;
 
 	stats_increment(ICA_STATS_AES_GCM,
-			hardware,
+			ALGO_HW,
 			(s390_kma_functions[fc].hw_fc &
 			S390_CRYPTO_DIRECTION_MASK) == 0 ?
 			ENCRYPT:DECRYPT);
 
-	return rc;
+	return 0;
 }
 
 static inline int __s390_aes_ctrlist(unsigned int fc, unsigned long data_length,
@@ -148,22 +151,20 @@ static inline int __s390_aes_ctrlist(unsigned int fc, unsigned long data_length,
 				     unsigned char *key,
 				     unsigned char *out_data)
 {
-	int rc = EPERM;
-	int hardware = ALGO_HW;
+	int rc = ENODEV;
 
 	if (*s390_msa4_functions[fc].enabled)
 		rc = s390_ctr_hw(s390_msa4_functions[fc].hw_fc,
 				 data_length, in_data, key,
 				 out_data, ctrlist);
-	if (rc) {
-		hardware = ALGO_SW;
+	if (rc)
 		return rc;
-	}
-	stats_increment(ICA_STATS_AES_CTR, hardware,
+
+	stats_increment(ICA_STATS_AES_CTR, ALGO_HW,
 			 (s390_msa4_functions[fc].hw_fc &
 			 S390_CRYPTO_DIRECTION_MASK) ==
 			 0 ?ENCRYPT:DECRYPT);
-	return rc;
+	return 0;
 }
 
 
@@ -303,6 +304,7 @@ static inline int s390_aes_ecb_sw(unsigned int function_code,
 				  unsigned char *output_data)
 {
 	AES_KEY aes_key;
+	unsigned long i;
 	unsigned int direction;
 	unsigned int key_size = (function_code & 0x0f) *
 				sizeof(ica_aes_key_single_t);
@@ -319,7 +321,7 @@ static inline int s390_aes_ecb_sw(unsigned int function_code,
 		AES_set_encrypt_key(keys, key_size * 8, &aes_key);
 		direction = AES_ENCRYPT;
 	}
-	int i;
+
 	for (i = 0; i < input_length; i += AES_BLOCK_SIZE) {
 		AES_ecb_encrypt(input_data + i, output_data + i,
 				&aes_key, direction);
@@ -354,8 +356,9 @@ static inline int s390_aes_cbc_hw(unsigned int function_code,
 	if (rc >= 0) {
 		memcpy(iv, &key_buffer.iv, sizeof(ica_aes_vector_t));
 		return 0;
-	} else
+	} else {
 		return EIO;
+	}
 }
 
 static inline int s390_aes_cbc_sw(unsigned int function_code,
@@ -392,7 +395,7 @@ static inline int s390_aes_ecb(unsigned int fc, unsigned long data_length,
 			const unsigned char *in_data, unsigned char *key,
 			unsigned char *out_data)
 {
-	int rc = 1;
+	int rc = ENODEV;
 	int hardware = ALGO_HW;
 
 	if (*s390_kmc_functions[fc].enabled)
@@ -400,6 +403,8 @@ static inline int s390_aes_ecb(unsigned int fc, unsigned long data_length,
 				     data_length, in_data, key,
 				     out_data);
 	if (rc) {
+		if (!ica_fallbacks_enabled)
+			return rc;
 		rc = s390_aes_ecb_sw(s390_kmc_functions[fc].hw_fc,
 				     data_length, in_data, key,
 				     out_data);
@@ -417,7 +422,7 @@ static inline int s390_aes_cbc(unsigned int fc, unsigned long data_length,
 			const unsigned char *in_data, unsigned char *iv,
 			unsigned char *key, unsigned char *out_data)
 {
-	int rc = 1;
+	int rc = ENODEV;
 	int hardware = ALGO_HW;
 
 	if (*s390_kmc_functions[fc].enabled)
@@ -425,14 +430,17 @@ static inline int s390_aes_cbc(unsigned int fc, unsigned long data_length,
 				     data_length, in_data, iv, key,
 				     out_data);
 	if (rc) {
-		hardware = ALGO_SW;
+		if (!ica_fallbacks_enabled)
+			return rc;
 		rc = s390_aes_cbc_sw(s390_kmc_functions[fc].hw_fc,
 				     data_length, in_data, iv, key,
 				     out_data);
+		hardware = ALGO_SW;
 	}
 	stats_increment(ICA_STATS_AES_CBC,
 			hardware, (s390_kmc_functions[fc].hw_fc &
-			S390_CRYPTO_DIRECTION_MASK) == 0 ?ENCRYPT:DECRYPT);
+			S390_CRYPTO_DIRECTION_MASK) == 0 ?
+			ENCRYPT:DECRYPT);
 	return rc;
 }
 
@@ -473,24 +481,21 @@ static inline int __s390_aes_cfb(unsigned int fc, unsigned long data_length,
 				 unsigned char *iv, const unsigned char *key,
 				 unsigned char *out_data, unsigned int lcfb)
 {
-	int rc = 1;
-	int hardware = ALGO_HW;
+	int rc = ENODEV;
 
 	if (*s390_msa4_functions[fc].enabled)
 		rc = s390_aes_cfb_hw(s390_msa4_functions[fc].hw_fc,
 				     data_length, in_data, iv, key,
 				     out_data, lcfb);
-	if (rc) {
-		hardware = ALGO_SW;
-		return EPERM;
-	}
+	if (rc)
+		return rc;
 
-	stats_increment(ICA_STATS_AES_CFB, hardware,
+	stats_increment(ICA_STATS_AES_CFB, ALGO_HW,
 			(s390_kmc_functions[fc].hw_fc &
 			S390_CRYPTO_DIRECTION_MASK) == 0 ?
 			ENCRYPT:DECRYPT);
 
-	return rc;
+	return 0;
 }
 
 static inline int s390_aes_cfb(unsigned int fc, unsigned long data_length,
@@ -569,23 +574,20 @@ static inline int __s390_aes_ofb(unsigned int fc, unsigned long input_length,
 				 unsigned char *iv, const unsigned char *keys,
 				 unsigned char *output_data)
 {
-	int rc = EPERM;
-	int hardware = ALGO_HW;
+	int rc = ENODEV;
 
 	if (*s390_msa4_functions[fc].enabled)
 		rc = s390_aes_ofb_hw(s390_msa4_functions[fc].hw_fc,
 				     input_length, input_data, iv, keys,
 				     output_data);
-	if (rc) {
-		hardware = ALGO_SW;
+	if (rc)
 		return rc;
-	}
 
-	stats_increment(ICA_STATS_AES_OFB, hardware,
+	stats_increment(ICA_STATS_AES_OFB, ALGO_HW,
 			(s390_kmc_functions[fc].hw_fc &
 			S390_CRYPTO_DIRECTION_MASK) == 0 ?
 			ENCRYPT:DECRYPT);
-	return rc;
+	return 0;
 }
 
 static inline int s390_aes_ofb(unsigned int fc, unsigned long data_length,
@@ -817,22 +819,20 @@ static inline int s390_aes_xts(unsigned int fc, unsigned long data_length,
 			unsigned char *key1, unsigned char *key2,
 			unsigned int key_length, unsigned char *out_data)
 {
-	int rc = 1;
-	int hardware = ALGO_HW;
+	int rc = ENODEV;
 
 	if (*s390_msa4_functions[fc].enabled)
 		rc = s390_aes_xts_hw(s390_msa4_functions[fc].hw_fc,
 				     data_length, in_data, tweak,
 				     key1, key2, key_length, out_data);
-	if (rc) {
-		hardware = ALGO_SW;
+	if (rc)
 		return rc;
-	}
 
-	stats_increment(ICA_STATS_AES_XTS, hardware,
+	stats_increment(ICA_STATS_AES_XTS, ALGO_HW,
 			(s390_kmc_functions[fc].hw_fc &
 			S390_CRYPTO_DIRECTION_MASK) == 0 ?
 			ENCRYPT:DECRYPT);
-	return rc;
+
+	return 0;
 }
 #endif
