@@ -5,8 +5,8 @@
  */
 
 /*
- * Entry point for crypto graphic instructions on s390. If a instruction
- * is not available, instruction will be simulated in software.
+ * Entry point for crypto graphic instructions on s390. If an instruction
+ * is not available, the related function is simulated in software.
  *
  * Authors(s): Ralph Wuerthner <rwuerthn@de.ibm.com>
  *	       Jan Glauber <jan.glauber@de.ibm.com>
@@ -24,16 +24,23 @@
 #include <stdio.h>
 #include <stdlib.h>
 #include <errno.h>
+#include <dirent.h>
 
 #include "fips.h"
 #include "init.h"
 #include "s390_crypto.h"
 
-
+unsigned long long facility_bits[3];
 unsigned int sha1_switch, sha256_switch, sha512_switch, sha3_switch, des_switch,
 	     tdes_switch, aes128_switch, aes192_switch, aes256_switch,
 	     prng_switch, tdea128_switch, tdea192_switch, sha512_drng_switch,
-	     msa4_switch, msa5_switch, msa8_switch, trng_switch;
+	     msa4_switch, msa5_switch, msa8_switch, trng_switch, msa9_switch,
+		 ecc_via_online_card, any_card_online;
+
+#define CARD_AVAILABLE		0x01
+#define CEXnA_AVAILABLE		0x02
+#define CEXnC_AVAILABLE		0x04
+#define CEX4C_AVAILABLE		0x08
 
 s390_supported_function_t s390_kimd_functions[] = {
 	{SHA_1, S390_CRYPTO_SHA_1, &sha1_switch},
@@ -47,7 +54,9 @@ s390_supported_function_t s390_kimd_functions[] = {
 	{SHA_3_512, S390_CRYPTO_SHA_3_512, &sha3_switch},
 	{SHAKE_128, S390_CRYPTO_SHAKE_128, &sha3_switch},
 	{SHAKE_256, S390_CRYPTO_SHAKE_256, &sha3_switch},
-	{GHASH, S390_CRYPTO_GHASH, &msa4_switch}
+	{GHASH, S390_CRYPTO_GHASH, &msa4_switch},
+	{SHA_512_224, S390_CRYPTO_SHA_512, &sha512_switch},
+	{SHA_512_256, S390_CRYPTO_SHA_512, &sha512_switch}
 };
 
 s390_supported_function_t s390_kmc_functions[] = {
@@ -85,6 +94,22 @@ s390_supported_function_t s390_msa4_functions[] = {
 	{AES_256_XTS_DECRYPT, S390_CRYPTO_AES_256_XTS_DECRYPT, &msa4_switch}
 };
 
+s390_supported_function_t s390_pcc_functions[] = {
+	{0, 0, &msa4_switch},	/* CMAC_AES not supported */
+	{0, 0, &msa4_switch},
+	{0, 0, &msa4_switch},
+	{0, 0, &msa4_switch},
+	{0, 0, &msa4_switch},
+	{0, 0, &msa4_switch},
+	{SCALAR_MULTIPLY_P256, S390_CRYPTO_SCALAR_MULTIPLY_P256, &msa9_switch},
+	{SCALAR_MULTIPLY_P384, S390_CRYPTO_SCALAR_MULTIPLY_P384, &msa9_switch},
+	{SCALAR_MULTIPLY_P521, S390_CRYPTO_SCALAR_MULTIPLY_P521, &msa9_switch},
+	{SCALAR_MULTIPLY_ED25519, S390_CRYPTO_SCALAR_MULTIPLY_ED25519, &msa9_switch},
+	{SCALAR_MULTIPLY_ED448, S390_CRYPTO_SCALAR_MULTIPLY_ED448, &msa9_switch},
+	{SCALAR_MULTIPLY_X25519, S390_CRYPTO_SCALAR_MULTIPLY_X25519, &msa9_switch},
+	{SCALAR_MULTIPLY_X448, S390_CRYPTO_SCALAR_MULTIPLY_X448, &msa9_switch},
+};
+
 s390_supported_function_t s390_ppno_functions[] = {
 	{SHA512_DRNG_GEN, S390_CRYPTO_SHA512_DRNG_GEN, &sha512_drng_switch},
 	{SHA512_DRNG_SEED, S390_CRYPTO_SHA512_DRNG_SEED, &sha512_drng_switch},
@@ -104,7 +129,20 @@ s390_supported_function_t s390_kma_functions[] = {
 	{AES_256_GCM_DECRYPT, S390_CRYPTO_AES_256_GCM_DECRYPT, &msa8_switch}
 };
 
-int read_cpuinfo(void)
+s390_supported_function_t s390_kdsa_functions[] = {
+	{ECDSA_VERIFY_P256, S390_CRYPTO_ECDSA_VERIFY_P256, &msa9_switch},
+	{ECDSA_VERIFY_P384, S390_CRYPTO_ECDSA_VERIFY_P384, &msa9_switch},
+	{ECDSA_VERIFY_P521, S390_CRYPTO_ECDSA_VERIFY_P521, &msa9_switch},
+	{ECDSA_SIGN_P256, S390_CRYPTO_ECDSA_SIGN_P256, &msa9_switch},
+	{ECDSA_SIGN_P384, S390_CRYPTO_ECDSA_SIGN_P384, &msa9_switch},
+	{ECDSA_SIGN_P521, S390_CRYPTO_ECDSA_SIGN_P521, &msa9_switch},
+	{EDDSA_VERIFY_ED25519, S390_CRYPTO_EDDSA_VERIFY_ED25519, &msa9_switch},
+	{EDDSA_VERIFY_ED448, S390_CRYPTO_EDDSA_VERIFY_ED448, &msa9_switch},
+	{EDDSA_SIGN_ED25519, S390_CRYPTO_EDDSA_SIGN_ED25519, &msa9_switch},
+	{EDDSA_SIGN_ED448, S390_CRYPTO_EDDSA_SIGN_ED448, &msa9_switch},
+};
+
+static int read_cpuinfo(void)
 {
 	int msa = 0;
 	FILE *handle = fopen("/proc/cpuinfo", "r");
@@ -123,19 +161,47 @@ int read_cpuinfo(void)
 	return msa;
 }
 
-int read_facility_bits(void)
+/*
+ * Check if "vector enablement control"-bit and
+ * "AFP register control"-bit in control register 0 are set.
+ */
+static int vx_enabled(void)
 {
+	FILE *fd;
+	char buf[4096];
+
+	if ((fd = fopen("/proc/cpuinfo", "r")) == NULL)
+		return 0;
+
+	buf[0] = '\0';
+
+	while ((fgets(buf, sizeof(buf), fd) != NULL)
+	       && (strstr(buf, "features") != buf));
+
+	fclose(fd);
+
+	return (strstr(buf, " vx ") != NULL) ? 1 : 0;
+}
+
+static int read_facility_bits(void)
+{
+	char *s;
+	int env_msa;
 	int msa = 0;
-	unsigned long long facility_bits[3] = {0};
 	struct sigaction oldact;
 	sigset_t oldset;
 	int rc = -1;
 
+	memset(&oldact, 0, sizeof(oldact));
+	memset(&oldset, 0, sizeof(oldset));
+	memset(facility_bits, 0, sizeof(facility_bits));
+
 	rc = begin_sigill_section(&oldact, &oldset);
-	if (!rc) {
+	if (!rc)
 		rc = __stfle(facility_bits, 3);
-		end_sigill_section(&oldact, &oldset);
-	}
+
+	end_sigill_section(&oldact, &oldset);
+
 	/* __stfle always returns the no. of double words needed to store the
 	 * facility bits. This quantity is machine dependent. With MSA8, we
 	 * need the first three double words. */
@@ -150,163 +216,265 @@ int read_facility_bits(void)
 			msa = 5;
 		if (facility_bits[2] & (1ULL << (191 - 146)))
 			msa = 8;
+		if (facility_bits[2] & (1ULL << (191 - 155)))
+			msa = 9;
 	}
 
 	/**
 	 * allow specifying the MSA level via environment variable
 	 * to simulate older hardware.
 	 */
-	char* s = getenv("MSA");
-	int env_msa;
-
+	s = getenv("MSA");
 	if (s) {
-#ifdef ICA_DEBUG
-		printf("msa from getenv: [%s] \n", s);
-#endif
 		if (sscanf(s, "%d", &env_msa) == 1)
 			msa = env_msa > msa ? msa : env_msa;
 	}
 
+	/* protect against disabled vector facility */
+	if (!vx_enabled()) {
+		facility_bits[2] &= ~(1ULL << 56 | 1ULL << 57 | 1ULL << 62);
+	}
+
 	return msa;
 }
 
-void set_switches(int msa)
+static void set_switches(int msa)
 {
 	unsigned char mask[16];
 	unsigned int n;
-	unsigned int on = 0;
 	struct sigaction oldact;
 	sigset_t oldset;
 
-	/* The function arrays contain integers. Thus to compute the amount of
-	 * their elements the result of sizeof(*functions) has to be divided by
-	 * sizeof(int).
-	 * The msa4_switch will be set in the kimd function. Because this is
-	 * the only switch for all MSA4 functions we just set it through the
-	 * kimd query and do not need to over the whole array. Therfore there
-	 * is also no distict setting of the switch needed in form
-	 * msa4_switch = 1. */
-
+	memset(&oldact, 0, sizeof(oldact));
+	memset(&oldset, 0, sizeof(oldset));
 
 	/* kmc query */
 	memset(mask, 0, sizeof(mask));
 	if (msa) {
-		if (begin_sigill_section(&oldact, &oldset) == 0) {
+		if (begin_sigill_section(&oldact, &oldset) == 0)
 			s390_kmc(S390_CRYPTO_QUERY, mask, (void *) 0, (void *) 0, 0);
-			end_sigill_section(&oldact, &oldset);
-		}
+		end_sigill_section(&oldact, &oldset);
 	}
 	for (n = 0; n < (sizeof(s390_kmc_functions) /
-			 sizeof(s390_supported_function_t)); n++) {
+			 sizeof(s390_supported_function_t)); n++)
 		if (S390_CRYPTO_TEST_MASK(mask, s390_kmc_functions[n].hw_fc))
-			on = 1;
-		else
-			on = 0;
-		*s390_kmc_functions[n].enabled = on;
-	}
+			*s390_kmc_functions[n].enabled = 1;
 
 	/* kimd query */
 	memset(mask, 0, sizeof(mask));
 	if (msa) {
-		if (begin_sigill_section(&oldact, &oldset) == 0) {
+		if (begin_sigill_section(&oldact, &oldset) == 0)
 			s390_kimd(S390_CRYPTO_QUERY, mask, (void *) 0, 0);
-			end_sigill_section(&oldact, &oldset);
-		}
+		end_sigill_section(&oldact, &oldset);
 	}
 	for (n = 0; n < (sizeof(s390_kimd_functions) /
-			 sizeof(s390_supported_function_t)); n++) {
+			 sizeof(s390_supported_function_t)); n++)
 		if (S390_CRYPTO_TEST_MASK(mask, s390_kimd_functions[n].hw_fc))
-			on = 1;
-		else
-			on = 0;
-		*s390_kimd_functions[n].enabled = on;
+			*s390_kimd_functions[n].enabled = 1;
+
+	/* pcc query */
+	memset(mask, 0, sizeof(mask));
+	if (4 <= msa) {
+		msa4_switch = 1;
+		if (begin_sigill_section(&oldact, &oldset) == 0)
+			s390_pcc(S390_CRYPTO_QUERY, mask);
+		end_sigill_section(&oldact, &oldset);
 	}
+	for (n = 0; n < (sizeof(s390_ppno_functions) /
+			 sizeof(s390_supported_function_t)); n++)
+		if (S390_CRYPTO_TEST_MASK(mask, s390_pcc_functions[n].hw_fc))
+			*s390_pcc_functions[n].enabled = 1;
 
 	/* ppno query */
 	memset(mask, 0, sizeof(mask));
 	if (5 <= msa) {
 		msa5_switch = 1;
-		if (begin_sigill_section(&oldact, &oldset) == 0) {
+		if (begin_sigill_section(&oldact, &oldset) == 0)
 			s390_ppno(S390_CRYPTO_QUERY, mask, NULL, 0, NULL, 0);
-			end_sigill_section(&oldact, &oldset);
-		}
+		end_sigill_section(&oldact, &oldset);
 	}
 	for (n = 0; n < (sizeof(s390_ppno_functions) /
-			 sizeof(s390_supported_function_t)); n++) {
+			 sizeof(s390_supported_function_t)); n++)
 		if (S390_CRYPTO_TEST_MASK(mask, s390_ppno_functions[n].hw_fc))
-			on = 1;
-		else
-			on = 0;
-		*s390_ppno_functions[n].enabled = on;
-	}
+			*s390_ppno_functions[n].enabled = 1;
 
 	/* kma query */
 	memset(mask, 0, sizeof(mask));
 	if (8 <= msa) {
 		msa8_switch = 1;
-		if (begin_sigill_section(&oldact, &oldset) == 0) {
+		if (begin_sigill_section(&oldact, &oldset) == 0)
 			s390_kma(S390_CRYPTO_QUERY, mask, NULL, NULL, 0, NULL, 0);
-			end_sigill_section(&oldact, &oldset);
-		}
+		end_sigill_section(&oldact, &oldset);
 	}
 	for (n = 0; n < (sizeof(s390_kma_functions) /
-			 sizeof(s390_supported_function_t)); n++) {
+			 sizeof(s390_supported_function_t)); n++)
 		if (S390_CRYPTO_TEST_MASK(mask, s390_kma_functions[n].hw_fc))
-			on = 1;
-		else
-			on = 0;
-		*s390_kma_functions[n].enabled = on;
+			*s390_kma_functions[n].enabled = 1;
+
+	/* kdsa query */
+	memset(mask, 0, sizeof(mask));
+	if (9 <= msa) {
+		msa9_switch = 1;
+		if (begin_sigill_section(&oldact, &oldset) == 0)
+			s390_kdsa(S390_CRYPTO_QUERY, mask, NULL, 0);
+		end_sigill_section(&oldact, &oldset);
 	}
+	for (n = 0; n < (sizeof(s390_kdsa_functions) /
+			 sizeof(s390_supported_function_t)); n++)
+		if (S390_CRYPTO_TEST_MASK(mask, s390_kdsa_functions[n].hw_fc))
+			*s390_kdsa_functions[n].enabled = 1;
+}
+
+unsigned int is_device_online(const char *dev)
+{
+	unsigned int ret = 0;
+	FILE *file;
+	char c;
+
+	if ((file = fopen(dev, "r")) == NULL)
+		return 0;
+
+	/* Check if device online: 0 = offline, 1 = online */
+	if ((c = fgetc(file)) == '1')
+		ret = 1;
+
+	fclose(file);
+
+	return ret;
+}
+
+unsigned int get_device_type(const char *dev, char *devtype)
+{
+	unsigned int ret = 0;
+	char *type = NULL;
+	size_t size = 0;
+	FILE *file;
+
+	file = fopen(dev, "r");
+	if (file == NULL)
+		return 0;
+
+	/* Read device type: e.g. 'CEX5C' */
+	if (getline(&type, &size, file) == -1)
+		goto end;
+
+	/* Did we get something like 'CEX5C\n'? */
+	if (strlen(type) != 6 || strncmp(type, "CEX", 3) != 0)
+		goto end;
+
+	memcpy(devtype, type, 5);
+	ret = 1;
+
+end:
+	free(type);
+	fclose(file);
+
+	return ret;
+}
+
+#define AP_PATH  "/sys/devices/ap"
+#define MAX_DEV_LEN 280
+
+unsigned int search_for_cards()
+{
+	DIR *sysDir;
+	unsigned int ret = 0;
+	char dev[MAX_DEV_LEN] = AP_PATH;
+	struct dirent *direntp;
+	char type[6];
+
+	if ((sysDir = opendir(dev)) == NULL)
+		return 0;
+
+	while ((direntp = readdir(sysDir)) != NULL) {
+
+		/* Skip entries that are not like "card01", "card02", etc. */
+		if (strncmp(direntp->d_name, "card", 4) != 0)
+			continue;
+
+		/* Check if device online */
+		snprintf(dev, MAX_DEV_LEN, "%s/%s/online", AP_PATH, direntp->d_name);
+		if (!is_device_online(dev))
+			continue;
+
+		/* Get device type (string like "CEXnT") */
+		snprintf(dev, MAX_DEV_LEN, "%s/%s/type", AP_PATH, direntp->d_name);
+		memset(type, 0, sizeof(type));
+		if (!get_device_type(dev, type))
+			continue;
+
+		/* Now setup return value according to found card */
+		if (type[4] == 'A')
+			ret |= CARD_AVAILABLE | CEXnA_AVAILABLE;
+
+		if (type[4] == 'C')
+			ret |= CARD_AVAILABLE | CEXnC_AVAILABLE;
+
+		if (type[3] >= '4' && type[4] == 'C')
+			ret |= CARD_AVAILABLE | CEX4C_AVAILABLE;
+	}
+
+	closedir(sysDir);
+
+	return ret;
 }
 
 void s390_crypto_switches_init(void)
 {
-	int msa;
+	int msa, flags;
 
 	msa = read_facility_bits();
 	if (!msa)
 		msa = read_cpuinfo();
 
+	flags = search_for_cards();
+	if (flags & CARD_AVAILABLE)
+		any_card_online = 1;
+	if (flags & CEX4C_AVAILABLE)
+		ecc_via_online_card = 1;
+
 	set_switches(msa);
 }
 
 /*
  * The first field represents the mechanism ID.
  * The second field represents the function family type (category),
- * The third filed represents the function code.
+ * The third field represents the function code.
  * This function code will be used later to check if HW support
  * is available and modifies the SW/HW-support-flag.
  * SHW - static hardware support (CPACF)
  * DHW - dynamic hardware support (crypto adapter)
  * SW  - software support
  * Bit field flags: [0|0|0|0|0|SHW|DHW|SW]
- * The last filed represent the property flags
+ * The last field represents the property flags indicating key lengths
  */
 libica_func_list_element_int icaList[] = {
- {SHA1,   KIMD, SHA_1  		, ICA_FLAG_SW, 0},
- {SHA224, KIMD, SHA_256		, ICA_FLAG_SW, 0},
- {SHA256, KIMD, SHA_256		, ICA_FLAG_SW, 0},
- {SHA384, KIMD, SHA_512		, ICA_FLAG_SW, 0},
- {SHA512, KIMD, SHA_512		, ICA_FLAG_SW, 0},
+ {SHA1,   KIMD, SHA_1  , ICA_FLAG_SW, 0},
+ {SHA224, KIMD, SHA_256, ICA_FLAG_SW, 0},
+ {SHA256, KIMD, SHA_256, ICA_FLAG_SW, 0},
+ {SHA384, KIMD, SHA_512, ICA_FLAG_SW, 0},
+ {SHA512, KIMD, SHA_512, ICA_FLAG_SW, 0},
+ {SHA512_224, KIMD, SHA_512_224, ICA_FLAG_SW, 0},
+ {SHA512_256, KIMD, SHA_512_256, ICA_FLAG_SW, 0},
  {SHA3_224, KIMD, SHA_3_224, 0, 0},
  {SHA3_256, KIMD, SHA_3_256, 0, 0},
  {SHA3_384, KIMD, SHA_3_384, 0, 0},
  {SHA3_512, KIMD, SHA_3_512, 0, 0},
  {SHAKE128, KIMD, SHAKE_128, 0, 0},
  {SHAKE256, KIMD, SHAKE_256, 0, 0},
- {G_HASH, KIMD, GHASH		,           0, 0},
+ {G_HASH, KIMD, GHASH, 0, 0},
 
  {DES_ECB,      KMC,  DEA_ENCRYPT, ICA_FLAG_SW, 0},
  {DES_CBC,      KMC,  DEA_ENCRYPT, ICA_FLAG_SW, 0},
  {DES_OFB,      MSA4, DEA_ENCRYPT, 0, 0},
  {DES_CFB,      MSA4, DEA_ENCRYPT, 0, 0},
  {DES_CTR,      MSA4, DEA_ENCRYPT, 0, 0},
- {DES_CMAC,     MSA4, DEA_ENCRYPT, 0, 0},				// CPACF only (MSA4)
+ {DES_CMAC,     MSA4, DEA_ENCRYPT, 0, 0}, // CPACF only (MSA4)
 
  {DES3_ECB,     KMC,  TDEA_192_ENCRYPT, ICA_FLAG_SW, 0},
  {DES3_CBC,     KMC,  TDEA_192_ENCRYPT, ICA_FLAG_SW, 0},
  {DES3_OFB,     MSA4, TDEA_192_ENCRYPT,           0, 0},
- {DES3_CFB,     MSA4, TDEA_192_ENCRYPT, 	  0, 0},
+ {DES3_CFB,     MSA4, TDEA_192_ENCRYPT,           0, 0},
  {DES3_CTR,     MSA4, TDEA_192_ENCRYPT,           0, 0},
  {DES3_CMAC,    MSA4, TDEA_192_ENCRYPT,           0, 0},
 
@@ -316,21 +484,32 @@ libica_func_list_element_int icaList[] = {
  {AES_CFB,      MSA4, AES_128_ENCRYPT,           0, 0},
  {AES_CTR,      MSA4, AES_128_ENCRYPT,           0, 0},
  {AES_CMAC,     MSA4, AES_128_ENCRYPT,           0, 0},
- {AES_CCM,      MSA4, AES_128_ENCRYPT, 	         0, 0},
+ {AES_CCM,      MSA4, AES_128_ENCRYPT,           0, 0},
  {AES_GCM,      MSA4, AES_128_ENCRYPT,           0, 0},
  {AES_GCM_KMA,  MSA8, AES_128_GCM_ENCRYPT,       0, 0},
  {AES_XTS,      MSA4, AES_128_XTS_ENCRYPT,       0, 0},
- {P_RNG,    		ADAPTER, 0, ICA_FLAG_SHW | ICA_FLAG_SW, 0},	// SHW (CPACF) + SW
- {RSA_ME, 		ADAPTER, 0, ICA_FLAG_DHW | ICA_FLAG_SW, 0x0F},	// DHW (CEX) + SW / 512,1024,2048, 4096 bit key length
- {RSA_CRT, 		ADAPTER, 0, ICA_FLAG_DHW | ICA_FLAG_SW, 0x0F},	// DHW (CEX) + SW / 512,1024,2048, 4096 bit key length
- {RSA_KEY_GEN_ME,  	ADAPTER, 0, ICA_FLAG_SW, 		0},	// SW (openssl)
- {RSA_KEY_GEN_CRT, 	ADAPTER, 0, ICA_FLAG_SW, 		0},	// SW (openssl)
+ {P_RNG,        ADAPTER, 0, ICA_FLAG_SHW | ICA_FLAG_SW, 0}, // SHW (CPACF) + SW
+ {EC_DH,        ADAPTER, 0, 0, 0},
+ {EC_DSA_SIGN,	ADAPTER, 0, 0, 0},
+ {EC_DSA_VERIFY, ADAPTER, 0, 0, 0},
+ {EC_KGEN,      ADAPTER, 0, 0, 0},
+ {ED25519_KEYGEN, MSA9, SCALAR_MULTIPLY_ED25519, 0, 0},
+ {ED25519_SIGN,   MSA9, EDDSA_SIGN_ED25519, 0, 0},
+ {ED25519_VERIFY, MSA9, EDDSA_VERIFY_ED25519, 0, 0},
+ {ED448_KEYGEN,   MSA9, SCALAR_MULTIPLY_ED448, 0, 0},
+ {ED448_SIGN,     MSA9, EDDSA_SIGN_ED448, 0, 0},
+ {ED448_VERIFY,   MSA9, EDDSA_VERIFY_ED448, 0, 0},
+ {X25519_KEYGEN,   MSA9, SCALAR_MULTIPLY_X25519, 0, 0},
+ {X25519_DERIVE,   MSA9, SCALAR_MULTIPLY_X25519, 0, 0},
+ {X448_KEYGEN,   MSA9, SCALAR_MULTIPLY_X448, 0, 0},
+ {X448_DERIVE,   MSA9, SCALAR_MULTIPLY_X448, 0, 0},
+ {RSA_ME,       ADAPTER, 0, 0, 0},
+ {RSA_CRT,      ADAPTER, 0, 0, 0},
+ {RSA_KEY_GEN_ME, ADAPTER, 0, ICA_FLAG_SW, 0},  // SW (openssl)
+ {RSA_KEY_GEN_CRT, ADAPTER, 0, ICA_FLAG_SW, 0}, // SW (openssl)
 
  {SHA512_DRNG, PPNO, SHA512_DRNG_GEN, ICA_FLAG_SW, 0},
 
-/* available for the MSA4 instruction */
-/* available for the RSA instruction */
-
 };
 
 /*
@@ -338,61 +517,94 @@ libica_func_list_element_int icaList[] = {
  * Query s390_xxx_functions for each algorithm to check
  * CPACF support and update the corresponding SHW-flags.
  */
-int s390_initialize_functionlist() {
+int s390_initialize_functionlist()
+{
+	unsigned int list_len = sizeof(icaList)/sizeof(libica_func_list_element_int);
+	unsigned int x;
+	
+	for (x = 0; x < list_len; x++) {
 
-  unsigned int list_len = sizeof(icaList)/sizeof(libica_func_list_element_int);
+		libica_func_list_element_int *e = &icaList[x];
 
-  unsigned int x;
-  for (x = 0; x < list_len; x++) {
-	switch ((int)icaList[x].type) {
-	case KIMD:
-		icaList[x].flags = icaList[x].flags |
-		    ((*s390_kimd_functions[icaList[x].id].enabled)? 4: 0);
-	break;
-	case KMC:
-		icaList[x].flags = icaList[x].flags |
-		((*s390_kmc_functions[icaList[x].id].enabled)? 4: 0);
-		if (icaList[x].id == AES_128_ENCRYPT) { // check for the maximum size
-		  if (*s390_kmc_functions[icaList[AES_256_ENCRYPT].id].enabled)
-			icaList[x].property = icaList[x].property | 4; // 256 bit
-		  if (*s390_kmc_functions[icaList[AES_192_ENCRYPT].id].enabled)
-			icaList[x].property = icaList[x].property | 2; // 192 bit
-		  if (*s390_kmc_functions[icaList[AES_128_ENCRYPT].id].enabled)
-			icaList[x].property = icaList[x].property | 1; // 128 bit
+		switch ((int) e->type) {
+		case KIMD:
+			e->flags |= *s390_kimd_functions[e->id].enabled ? ICA_FLAG_SHW : 0;
+			break;
+		case KMC:
+			e->flags |= *s390_kmc_functions[e->id].enabled ? ICA_FLAG_SHW : 0;
+			if (e->id == AES_128_ENCRYPT) { // check for the maximum size
+				if (*s390_kmc_functions[icaList[AES_256_ENCRYPT].id].enabled)
+					e->property |= ICA_PROPERTY_AES_256;
+				if (*s390_kmc_functions[icaList[AES_192_ENCRYPT].id].enabled)
+					e->property |= ICA_PROPERTY_AES_192;
+				if (*s390_kmc_functions[icaList[AES_128_ENCRYPT].id].enabled)
+					e->property |= ICA_PROPERTY_AES_128;
+			}
+			break;
+		case MSA4:
+			e->flags |= *s390_msa4_functions[e->id].enabled ? ICA_FLAG_SHW : 0;
+			if (e->id == AES_128_ENCRYPT) { // check for the maximum size
+				if (*s390_msa4_functions[icaList[AES_256_ENCRYPT].id].enabled)
+					e->property |= ICA_PROPERTY_AES_256;
+				if (*s390_msa4_functions[icaList[AES_192_ENCRYPT].id].enabled)
+					e->property |= ICA_PROPERTY_AES_192;
+				if (*s390_msa4_functions[icaList[AES_128_ENCRYPT].id].enabled)
+					e->property |= ICA_PROPERTY_AES_128;
+			} else if (e->id == AES_128_XTS_ENCRYPT) { // check for the maximum size
+				if (*s390_msa4_functions[icaList[AES_256_XTS_ENCRYPT].id].enabled)
+					e->property |= ICA_PROPERTY_AES_256;
+				if (*s390_msa4_functions[icaList[AES_128_XTS_ENCRYPT].id].enabled)
+					e->property |= ICA_PROPERTY_AES_128;
+			}
+			break;
+		case PPNO:
+			e->flags |= *s390_ppno_functions[e->id].enabled ? ICA_FLAG_SHW : 0;
+			break;
+		case MSA8:
+			e->flags |= *s390_kma_functions[e->id].enabled ? ICA_FLAG_SHW : 0;
+			break;
+		case MSA9:
+			if (e->mech_mode_id == ED25519_KEYGEN
+				|| e->mech_mode_id == ED448_KEYGEN
+				|| e->mech_mode_id == X25519_KEYGEN
+				|| e->mech_mode_id == X25519_DERIVE
+				|| e->mech_mode_id == X448_KEYGEN
+				|| e->mech_mode_id == X448_DERIVE)
+				e->flags |= *s390_pcc_functions[e->id].enabled ? ICA_FLAG_SHW : 0;
+			else
+				e->flags |= *s390_kdsa_functions[e->id].enabled ? ICA_FLAG_SHW : 0;
+			e->property |= ICA_PROPERTY_EC_ED;
+			break;
+		default:
+			/* Do nothing. */
+			break;
 		}
-	break;
-	case MSA4:
-		icaList[x].flags = icaList[x].flags |
-		((*s390_msa4_functions[icaList[x].id].enabled)? 4: 0);
-		  if (icaList[x].id == AES_128_ENCRYPT) { // check for the maximum size
-			if (*s390_msa4_functions[icaList[AES_256_ENCRYPT].id].enabled)
-				icaList[x].property = icaList[x].property | 4; // 256 bit
-			if (*s390_msa4_functions[icaList[AES_192_ENCRYPT].id].enabled)
-				icaList[x].property = icaList[x].property | 2; // 192 bit
-			if (*s390_msa4_functions[icaList[AES_128_ENCRYPT].id].enabled)
-				icaList[x].property = icaList[x].property | 1; // 128 bit
-		  }
-		  else if (icaList[x].id == AES_128_XTS_ENCRYPT) { // check for the maximum size
-			if (*s390_msa4_functions[icaList[AES_256_XTS_ENCRYPT].id].enabled)
-				icaList[x].property = icaList[x].property | 2; // 256 bit
-			if (*s390_msa4_functions[icaList[AES_128_XTS_ENCRYPT].id].enabled)
-				icaList[x].property = icaList[x].property | 1; // 128 bit
+
+		switch ((int) e->mech_mode_id) {
+		case EC_DH: /* fall-through */
+		case EC_DSA_SIGN: /* fall-through */
+		case EC_DSA_VERIFY: /* fall-through */
+		case EC_KGEN:
+			if (ecc_via_online_card) {
+				e->flags |= ICA_FLAG_DHW;
+				e->property |= ICA_PROPERTY_EC_BP | ICA_PROPERTY_EC_NIST;
+			}
+			e->flags |= *s390_kdsa_functions[e->id].enabled ? ICA_FLAG_SHW : 0;
+			break;
+		case RSA_ME: /* fall-through */
+		case RSA_CRT:
+			if (any_card_online) {
+				e->flags |= ICA_FLAG_DHW;
+				e->property |= ICA_PROPERTY_RSA_ALL;
+			}
+			break;
+		default:
+			/* Do nothing. */
+			break;
 		}
-	break;
-	case PPNO:
-		icaList[x].flags = icaList[x].flags |
-		    ((*s390_ppno_functions[icaList[x].id].enabled)? 4: 0);
-	break;
-	case MSA8:
-		icaList[x].flags = icaList[x].flags |
-		((*s390_kma_functions[icaList[x].id].enabled)? 4: 0);
-	break;
-	default:
-		/* Do nothing. */
-	break;
 	}
-  }
-  return 0;
+
+	return 0;
 }
 
 /**
@@ -425,7 +637,7 @@ int s390_initialize_functionlist() {
  */
 int s390_get_functionlist(libica_func_list_element *pmech_list,
 				      unsigned int *pmech_list_len) {
-  int x;
+  unsigned int x;
 
   if (!pmech_list_len) {
 	return EINVAL;
@@ -434,13 +646,12 @@ int s390_get_functionlist(libica_func_list_element *pmech_list,
   if (!pmech_list) {
 	*pmech_list_len = sizeof(icaList)/sizeof(libica_func_list_element_int);
 	return 0;
-  }
-  else if (*pmech_list_len <
-	  (sizeof(icaList)/sizeof(libica_func_list_element_int)) ) {
+  } else if (*pmech_list_len <
+	     (sizeof(icaList)/sizeof(libica_func_list_element_int)) ) {
 	return EINVAL;
   }
 
-  for (x=0; x<*pmech_list_len; x++) {
+  for (x = 0; x < *pmech_list_len; x++) {
       pmech_list[x].mech_mode_id = icaList[x].mech_mode_id;
       pmech_list[x].flags        = icaList[x].flags;
       pmech_list[x].property     = icaList[x].property;
@@ -457,6 +668,5 @@ int s390_get_functionlist(libica_func_list_element *pmech_list,
 	}
 #endif /* ICA_FIPS */
   }
-
   return 0;
 }
